@@ -15,9 +15,6 @@
 
 //! Integration tests that use golden input and output files.
 
-// TODO(jmmv): All of the supporting code here is duplicated in the other EndBASIC crates.
-// Should probably add a separate testutils crate, but I'm hesitant for now.
-
 // Keep these in sync with other top-level files.
 #![warn(anonymous_parameters, bad_style, missing_docs)]
 #![warn(unused, unused_extern_crates, unused_import_braces, unused_qualifications)]
@@ -168,6 +165,24 @@ fn check<P: AsRef<Path>>(
     }
 }
 
+#[test]
+fn test_cli_autoexec_is_ignored() {
+    let dir = tempfile::tempdir().unwrap();
+    fs::copy(&src_path("cli/tests/repl/autoexec.bas"), &dir.path().join("AUTOEXEC.BAS")).unwrap();
+    check(
+        bin_path("endbasic"),
+        &[
+            "--programs-dir",
+            dir.path().to_str().unwrap(),
+            &src_str("cli/tests/cli/interactive.bas"),
+        ],
+        1,
+        Behavior::Null,
+        Behavior::Null,
+        Behavior::File(src_path("cli/tests/cli/interactive.err")),
+    );
+}
+
 #[test]
 fn test_cli_help() {
     fn check_with_args(args: &[&str]) {
@@ -186,6 +201,36 @@ fn test_cli_help() {
     check_with_args(&["the", "--help", "flag always wins"]);
 }
 
+#[test]
+fn test_cli_interactive() {
+    check(
+        bin_path("endbasic"),
+        &[&src_str("cli/tests/cli/interactive.bas")],
+        1,
+        Behavior::Null,
+        Behavior::Null,
+        Behavior::File(src_path("cli/tests/cli/interactive.err")),
+    );
+
+    check(
+        bin_path("endbasic"),
+        &["-i", &src_str("cli/tests/cli/interactive.bas")],
+        0,
+        Behavior::Null,
+        Behavior::File(src_path("cli/tests/cli/interactive.out")),
+        Behavior::Null,
+    );
+
+    check(
+        bin_path("endbasic"),
+        &["--interactive", &src_str("cli/tests/cli/interactive.bas")],
+        0,
+        Behavior::Null,
+        Behavior::File(src_path("cli/tests/cli/interactive.out")),
+        Behavior::Null,
+    );
+}
+
 // TODO(jmmv): This test fails almost always on Linux CI builds with `Text file busy` when
 // attempting to run the copied binary.  I've also gotten it to occasionally fail on a local Linux
 // installation in the same way, but that's much harder to trigger.  Investigate what's going on.
@@ -265,6 +310,152 @@ fn test_cli_version() {
     check_with_args(&["the", "--version", "flag wins over arguments"]);
 }
 
+#[test]
+fn test_example_guess() {
+    check(
+        bin_path("endbasic"),
+        &["--interactive", &src_str("cli/tests/examples/guess.bas")],
+        0,
+        Behavior::File(src_path("cli/tests/examples/guess.in")),
+        Behavior::File(src_path("cli/tests/examples/guess.out")),
+        Behavior::Null,
+    );
+}
+
+#[test]
+fn test_example_hello() {
+    check(
+        bin_path("endbasic"),
+        &["--interactive", &src_str("cli/tests/examples/hello.bas")],
+        0,
+        Behavior::File(src_path("cli/tests/examples/hello.in")),
+        Behavior::File(src_path("cli/tests/examples/hello.out")),
+        Behavior::Null,
+    );
+}
+
+#[test]
+fn test_example_tour() {
+    check(
+        bin_path("endbasic"),
+        &["--interactive", &src_str("cli/tests/examples/tour.bas")],
+        0,
+        Behavior::File(src_path("cli/tests/examples/tour.in")),
+        Behavior::File(src_path("cli/tests/examples/tour.out")),
+        Behavior::Null,
+    );
+}
+
+#[test]
+fn test_lang_control_flow() {
+    check(
+        bin_path("endbasic"),
+        &[&src_str("cli/tests/lang/control-flow.bas")],
+        0,
+        Behavior::Null,
+        Behavior::File(src_path("cli/tests/lang/control-flow.out")),
+        Behavior::Null,
+    );
+}
+
+#[test]
+fn test_lang_exec_error() {
+    check(
+        bin_path("endbasic"),
+        &[&src_str("cli/tests/lang/exec-error.bas")],
+        1,
+        Behavior::Null,
+        Behavior::File(src_path("cli/tests/lang/exec-error.out")),
+        Behavior::File(src_path("cli/tests/lang/exec-error.err")),
+    );
+}
+
+#[test]
+fn test_lang_hello() {
+    check(
+        bin_path("endbasic"),
+        &[&src_str("cli/tests/lang/hello.bas")],
+        0,
+        Behavior::Null,
+        Behavior::File(src_path("cli/tests/lang/hello.out")),
+        Behavior::Null,
+    );
+}
+
+#[test]
+fn test_lang_lexer_error() {
+    check(
+        bin_path("endbasic"),
+        &[&src_str("cli/tests/lang/lexer-error.bas")],
+        1,
+        Behavior::Null,
+        Behavior::File(src_path("cli/tests/lang/lexer-error.out")),
+        Behavior::File(src_path("cli/tests/lang/lexer-error.err")),
+    );
+}
+
+#[test]
+fn test_lang_parser_error() {
+    check(
+        bin_path("endbasic"),
+        &[&src_str("cli/tests/lang/parser-error.bas")],
+        1,
+        Behavior::Null,
+        Behavior::File(src_path("cli/tests/lang/parser-error.out")),
+        Behavior::File(src_path("cli/tests/lang/parser-error.err")),
+    );
+}
+
+#[test]
+fn test_lang_types() {
+    check(
+        bin_path("endbasic"),
+        &[&src_str("cli/tests/lang/types.bas")],
+        0,
+        Behavior::Null,
+        Behavior::File(src_path("cli/tests/lang/types.out")),
+        Behavior::Null,
+    );
+}
+
+#[test]
+fn test_lang_utf8() {
+    check(
+        bin_path("endbasic"),
+        &[&src_str("cli/tests/lang/utf8.bas")],
+        0,
+        Behavior::File(src_path("cli/tests/lang/utf8.in")),
+        Behavior::File(src_path("cli/tests/lang/utf8.out")),
+        Behavior::Null,
+    );
+}
+
+#[test]
+fn test_lang_yes_no() {
+    check(
+        bin_path("endbasic"),
+        &[&src_str("cli/tests/lang/yes-no.bas")],
+        0,
+        Behavior::File(src_path("cli/tests/lang/yes-no.in")),
+        Behavior::File(src_path("cli/tests/lang/yes-no.out")),
+        Behavior::Null,
+    );
+}
+
+#[test]
+fn test_repl_autoexec() {
+    let dir = tempfile::tempdir().unwrap();
+    fs::copy(&src_path("cli/tests/repl/autoexec.bas"), &dir.path().join("AUTOEXEC.BAS")).unwrap();
+    check(
+        bin_path("endbasic"),
+        &["--programs-dir", dir.path().to_str().unwrap()],
+        0,
+        Behavior::File(src_path("cli/tests/repl/hello.bas")),
+        Behavior::File(src_path("cli/tests/repl/autoexec.out")),
+        Behavior::Null,
+    );
+}
+
 #[test]
 fn test_repl_colors() {
     check(
@@ -291,11 +482,9 @@ fn test_repl_console() {
 
 #[test]
 fn test_repl_dir() {
-    let dir = tempfile::tempdir().unwrap();
-    let subdir = dir.path().join("subdir"); // Start with a non-existent directory.
     check(
         bin_path("endbasic"),
-        &["--programs-dir", subdir.to_str().unwrap()],
+        &["--programs-dir=:memory:"],
         0,
         Behavior::File(src_path("cli/tests/repl/dir.in")),
         Behavior::File(src_path("cli/tests/repl/dir.out")),
@@ -316,10 +505,10 @@ fn test_repl_editor() {
 }
 
 #[test]
-fn test_repl_exit() {
+fn test_repl_exit_nonzero() {
     check(
         bin_path("endbasic"),
-        &[&src_str("cli/tests/repl/exit.bas")],
+        &[&src_str("cli/tests/repl/exit-nonzero.bas")],
         78,
         Behavior::Null,
         Behavior::Null,
@@ -330,7 +519,28 @@ fn test_repl_exit() {
         bin_path("endbasic"),
         &[],
         78,
-        Behavior::File(src_path("cli/tests/repl/exit.bas")),
+        Behavior::File(src_path("cli/tests/repl/exit-nonzero.bas")),
+        Behavior::File(src_path("cli/tests/repl/exit.out")),
+        Behavior::Null,
+    );
+}
+
+#[test]
+fn test_repl_exit_zero() {
+    check(
+        bin_path("endbasic"),
+        &[&src_str("cli/tests/repl/exit-zero.bas")],
+        0,
+        Behavior::Null,
+        Behavior::Null,
+        Behavior::Null,
+    );
+
+    check(
+        bin_path("endbasic"),
+        &[],
+        0,
+        Behavior::File(src_path("cli/tests/repl/exit-zero.bas")),
         Behavior::File(src_path("cli/tests/repl/exit.out")),
         Behavior::Null,
     );
